Attribute VB_Name = "gLcd"
Option Explicit

'gLCD-Ansteuerung von Christof Rue, www.hobby-elektronik.de.vu
'Die Verwendung dieses Moduls ist kostenfrei, solange diese Zeilen erhalten bleiben.
'Fragen, Kritik und Anregungen an: hobbyelektronik@gmx.net
'Groer Dank gilt bei diesem Modul Martin K (martin-k(at)xmail(dot)net), der sehr groe Vorarbeit geleistet hat
'Die Anschlussbelegung ist der oben genannten Homepage zu entnehmen...

'Deklaration fr die Portansteuerung (auch fr Win2k & WinXP)
'Public Declare Sub Portout32 Lib "inpout32.dll" Alias "Out32" (ByVal PortAddress As Integer, ByVal Value As Integer)
Public Declare Function PortIn32 Lib "inpout32.dll" Alias "Inp32" (ByVal PortAddress As Integer) As Integer
Public Declare Sub Portout32 Lib "dlportio.dll" Alias "DlPortWritePortUlong" (ByVal Port As Long, ByVal Value As Long)


'Falls QueryPerformance* nicht verfgbar ist, wird das normale Sleep verwendet.
Public Declare Sub Sleep Lib "kernel32" (ByVal dwMilliSeconds As Long)

'Damit das Auslesen der Bilder fr die selbstdefinierten Zeichen schneller geht, bediene ich mich dieser API.
Public Declare Function GetPixel Lib "gdi32" (ByVal hDC As Long, ByVal x As Long, ByVal Y As Long) As Long
Public Declare Function SetPixel Lib "gdi32" (ByVal hDC As Long, ByVal x As Long, ByVal Y As Long, ByVal crColor As Long) As Long

'Zum Testen der bertragungsgeschwindigkeit notwendig)
Public Declare Function QueryPerformanceCounter Lib "kernel32" (lpPerformanceCount As Currency) As Long
Public Declare Function QueryPerformanceFrequency Lib "kernel32" (lpFrequency As Currency) As Long


Const SCLhi = 1
Const SCLlo = 0
Const SIOhi = 2
Const SIOlo = 0
Const CShi = 4
Const CSlo = 0
Const REShi = 8
Const RESlo = 0 'kann eigentlich auch 8 sein, da Reset auch dauerhaft auf High liegen kann

Private intLastBit As Long
Private LcdPort As Long
Private intContrast As Long
Private intHighColorDepth As Boolean
Private intInvert As Boolean
Private intDisplayOn As Boolean
Private intMirrored As Boolean
Private intFlipped As Boolean

'Hiermit wird der Port, an dem das LCD angschlossen ist festgelegt
Property Let Port(NewPort As Long)
LcdPort = NewPort
End Property

'Damit der Port auch gelesen werden kann, ist dies hier notwendig:
Property Get Port() As Long
Port = LcdPort
End Property

'Setzen des Display-Kontrasts. Der Wert muss zwischen 0 und 127 liegen. Alles andere wird ignoriert.
Property Let Contrast(NewContrast As Long)
If NewContrast <= 127 And NewContrast >= 0 Then
  intContrast = NewContrast
  LcdOut &H25, True
  LcdOut intContrast, False
Else
  intContrast = 70
End If
End Property

'Rckgabe
Property Get Contrast() As Long
Contrast = intContrast
End Property

'Festlegen der Farbtiefe
Property Let HighColorDepth(ByVal HighDepth As Boolean)
LcdOut &H3A, True
intHighColorDepth = HighDepth
If HighDepth = False Then
  LcdOut 2, False
Else
  LcdOut 3, False
End If
End Property

'Rckgabe
Property Get HighColorDepth() As Boolean
HighColorDepth = intHighColorDepth
End Property

'Display invertieren?
Property Let Inverted(ByVal isInverted As Boolean)
If isInverted = True Then
  LcdOut &H21, True
Else
  LcdOut &H20, True
End If
intInvert = isInverted
End Property

'Rckgabe
Property Get Inverted() As Boolean
Inverted = intInvert
End Property

'Display an- oder ausschalten
Property Let DisplayOn(ByVal LcdOn As Boolean)
intDisplayOn = LcdOn
If LcdOn = True Then
  LcdOut &H29, True
Else
  LcdOut &H28, True
End If
End Property

'Rckgabe
Property Get DisplayOn() As Boolean
DisplayOn = intDisplayOn
End Property

'Display horizontal gespiegelt?
Property Let Flipped(ByVal Flip As Boolean)
intFlipped = Flip
LcdOut &H36, True
LcdOut IIf(intFlipped = True, 128, 0) + IIf(intMirrored = True, 64, 0), False
End Property

'Rckgabe
Property Get Flipped() As Boolean
Flipped = intFlipped
End Property

'Display vertikal gespiegelt?
Property Let Mirrored(ByVal Mirror As Boolean)
intMirrored = Mirror
LcdOut &H36, True
LcdOut IIf(intFlipped = True, 128, 0) + IIf(intMirrored = True, 64, 0), False
End Property

'Rckgabe
Property Get Mirrored() As Boolean
Mirrored = intMirrored
End Property

'binren String in dezimalen Long umwandeln
Function BintoDez(ByVal Bin As String) As Long
  Dim x&, Y&
    Bin = Bin & String$(8 - Len(Bin), "0")
    For x = 1 To Len(Bin)
      If Mid$(Bin, x, 1) = "1" Then
        Y = Y + 2 ^ (8 - x)
      End If
    Next x
    BintoDez = Y
End Function

'dezimal -> binr
Function DeztoBin(ByVal Dez As Long) As String
  Dim x%
   If Dez >= 2 ^ 32 Then
     Call MsgBox("Zahl ist grer als 32 Bit")
     Exit Function
   End If
   Do
     If (Dez And 2 ^ x) Then
       DeztoBin = "1" & DeztoBin
     Else
       DeztoBin = "0" & DeztoBin
     End If
     x = x + 1
   Loop Until 2 ^ x > Dez
   DeztoBin = Format(DeztoBin, "00000000")
End Function

Function Init()
Dim x As Long
'Initialisierung RESET
Portout32 LcdPort, RESlo + SIOlo + SCLlo + CSlo
Portout32 LcdPort, REShi + SIOlo + SCLlo + CSlo
Sleep 5
Portout32 LcdPort, REShi + SIOhi + SCLhi + CShi
LcdOut &H1, True 'Software Reset
Sleep 5 '5ms warten
LcdOut &HC6, True 'Initial escape

'Display Setup 1

LcdOut &H9, True 'Displaystatus "lesen"
LcdOut 0, False

LcdOut &HB6, True 'Display Control
LcdOut 128, False '128 number of clocks, willkrliche Zahl
LcdOut 128, False 'no N line inversion (ungerade Zeilenanzahl)
LcdOut 129, False '2 divisions in idle mode, bias 1/9
LcdOut 84, False
LcdOut 69, False
LcdOut 82, False
LcdOut 67, False

LcdOut &HB3, True 'Gray Scale Position
LcdOut 1, False   'Bei Display Control wurden 128 clock numbers pro Zeile eingestellt
LcdOut 2, False   'Auf diese 128 Cyclen werden die 16 (4 bit) Graustufen verteilt
LcdOut 4, False   '(willkrlich, Werte mssen aber kleiner als 128 sein)
LcdOut 8, False   'Die Graustufen gelten fr alle Farben
LcdOut 16, False
LcdOut 30, False
LcdOut 40, False
LcdOut 50, False
LcdOut 60, False
LcdOut 70, False
LcdOut 80, False
LcdOut 90, False
LcdOut 100, False
LcdOut 110, False
LcdOut 127, False

LcdOut &HB5, True 'Gammakurve einstellen
LcdOut 1, False 'kann auch 2 sein, Unterschied ist ansich keiner da...

LcdOut &HBD, True 'Common Driver Output select
LcdOut 0, False 'eigentlich auch egal, einfach auf 0 lassen :)

'Power supply Setup

LcdOut &HBE, True 'Power control
LcdOut 4, False 'boosting clock = halbe display clock, interner Widerstand, high power Modus

LcdOut &H11, True 'Sleep out - nicht mehr schlafen!

LcdOut &HB7, True 'Spannungsverlauf der Temperatur anpassen
For x = 1 To 14   'nur erster Wert relevant, restliche 13 Werte auf 0 setzen
  LcdOut 0, False
Next x

LcdOut &H3, True 'Boosterspannung aktivieren
Sleep 40 '40ms warten, damit die Boosterspannung stabilisiert werden kann

'Display Setup 2

LcdOut &H20, True 'Display nicht invertieren

'Display Setup 3

LcdOut &H3A, True 'Farbtiefe einstellen
LcdOut 3, False 'auf 12 Bit
intHighColorDepth = True 'Fr WritePic die Farbtiefe auf 12 Bit stellen, sonst gibts rger

ColorSet 'Farbpalette fr 8-Bit-Betrieb setzen

LcdOut 0, False 'Bildorientierung einstellen: nicht gespiegelt
LcdOut &H29, True 'Display einschalten
End Function

Function Clear()
Dim Y As Long, x As Long
LcdOut &H2C, True
For Y = 1 To 67
  For x = 1 To 49
   LcdOut 0, False
   LcdOut 0, False
   LcdOut 0, False
  Next x
Next Y
End Function

'Daten ausgeben
Function LcdOut(ByVal LcdData As Byte, ByVal Command As Boolean)
Dim x As Long

If Command = True Then
  Portout32 LcdPort, CShi + intLastBit + REShi 'Wenn es ein Command ist, muss CS vorher noch auf hi gesetzt werden
End If

Portout32 LcdPort, CSlo + intLastBit + REShi 'Startbit

If Command = True Then
  Portout32 LcdPort, CSlo + SCLlo + SIOlo + REShi
  Portout32 LcdPort, CSlo + SCLhi + SIOlo + REShi
Else
  Portout32 LcdPort, CSlo + SCLlo + SIOhi + REShi
  Portout32 LcdPort, CSlo + SCLhi + SIOhi + REShi
End If

For x = 0 To 7
    If LcdData And 2 ^ (7 - x) Then
      Portout32 LcdPort, CSlo + SCLlo + SIOhi + REShi
      Portout32 LcdPort, CSlo + SCLhi + SIOhi + REShi
      intLastBit = SIOhi + SCLhi
    Else
      Portout32 LcdPort, CSlo + SCLlo + SIOlo + REShi
      Portout32 LcdPort, CSlo + SCLhi + SIOlo + REShi
      intLastBit = SIOlo + SCLhi
    End If
Next
End Function

'bertrgt einen vollstndigen Displayinhalt (98x67px) an das Display
'Die Formeln sehen etwas komisch aus, haben aber ihre berechtigung, da sonst die Farbwerte
'aufwendig zerlegt werden mssen. Die Pixel werden wie folgt an das Display bertragen:
' RRRRGGGG - RRRR: Rotanteil des 1. Pixels;   GGGG: Grnanteil des 1. Pixels
' BBBBRRRR - BBBB: Blauanteil des 1. Pixels;  RRRR: Rotanteil des 2. Pixels
' GGGGBBBB - GGGG: Grnanteil des 2. Pixels;  BBBB: Blauanteil des 2. Pixels
Function WritePic(ByVal LcdPicture As PictureBox)
Dim Cola As Long, Colb As Long 'Keine Schleichwerbung, wirklich nicht
Dim Ra As Long, Ga As Long, Ba As Long, Rb As Long, Gb As Long, Bb As Long
Dim Xval, Yval
Dim OldScaleMode As ScaleModeConstants
OldScaleMode = LcdPicture.ScaleMode 'Scalemode merken
LcdPicture.ScaleMode = 3 'Scalemode auf Pixel stellen
LcdOut &H2C, True
If intHighColorDepth = True Then
  For Yval = 0 To 66
    For Xval = 0 To 48
      Cola = 16777215 - GetPixel(LcdPicture.hDC, Xval * 2, Yval)
      Colb = 16777215 - GetPixel(LcdPicture.hDC, Xval * 2 + 1, Yval)
      Ra = Fix((Cola And 255) / 16) * 16
      Ga = Fix((Cola And 65025) / 4080)
      Ba = Fix((Cola And 16581375) / 1048576) * 16
      Rb = Fix((Colb And 255) / 16)
      Gb = Fix((Colb And 65025) / 4080) * 16
      Bb = Fix((Colb And 16581375) / 1048576)
      
      LcdOut Ra + Ga, False
      LcdOut Ba + Rb, False
      LcdOut Gb + Bb, False
    Next
  Next
Else
  For Yval = 0 To 66
    For Xval = 0 To 97
      Cola = 16777215 - GetPixel(LcdPicture.hDC, Xval, Yval)
      Ra = Fix((Cola And 255) / 32) * 32
      Ga = Fix((Cola And 65280) / 8192) * 4
      Ba = Round((Cola And 16711680) / 5570560)
      
      LcdOut Ra + Ga + Ba, False
    Next
  Next
End If
LcdPicture.ScaleMode = OldScaleMode
End Function

'Legt die Farbpalette fr die Ansteuerung im 8-Bit-Modus fest
'Nur Bit 0-3 sind interessant, wegen der Farbtiefe.
Function ColorSet()
LcdOut &H2D, True
'Tabelle fr Rot
LcdOut 0, False
LcdOut 2, False
LcdOut 4, False
LcdOut 6, False
LcdOut 8, False
LcdOut 10, False
LcdOut 12, False
LcdOut 14, False
'Tabelle fr Grn
LcdOut 0, False
LcdOut 2, False
LcdOut 4, False
LcdOut 6, False
LcdOut 8, False
LcdOut 10, False
LcdOut 12, False
LcdOut 14, False
'Tabelle fr Blau
LcdOut 0, False
LcdOut 4, False
LcdOut 8, False
LcdOut 12, False
End Function

